interface LicenseInfo {
  /** Requires attribution */
  attribution: boolean;
  /** Allows commercial use */
  commercial: boolean;
  /** Keep same license */
  sameLicense?: boolean;
}
/**
 * Data for open source licenses used by icon sets in `@iconify/json` package and smaller packages
 *
 * Key is SPDX license identifier
 */
declare const licensesData: Record<string, LicenseInfo>;
export { LicenseInfo, licensesData };