import{N as s,U as m,Z as c,_ as l,a as e,b as i,c as n,d as o,e as u,f as S}from"./chunk-AFFP7ECJ.mjs";var r=class extends l{static{e(this,"RadarTokenBuilder")}constructor(){super(["radar-beta"])}};var v={parser:{TokenBuilder:e(()=>new r,"TokenBuilder"),ValueConverter:e(()=>new c,"ValueConverter")}};function x(d=u){let a=o(n(d),s),t=o(i({shared:a}),m,v);return a.ServiceRegistry.register(t),{shared:a,Radar:t}}e(x,"createRadarServices");export{v as a,x as b};
