export namespace arrows {
    export { normal };
    export { vee };
    export { undirected };
}
export function setArrows(value: any): void;
declare function normal(parent: any, id: any, edge: any, type: any): void;
declare function vee(parent: any, id: any, edge: any, type: any): void;
declare function undirected(parent: any, id: any, edge: any, type: any): void;
export {};
