/**
 * @param {URL} url
 * @param {{parentURL: string}} context
 * @returns {string | null}
 */
export function defaultGetFormatWithoutErrors(url: URL, context: {
    parentURL: string;
}): string | null;
export type ProtocolHandler = (parsed: URL, context: {
    parentURL: string;
    source?: Buffer;
}, ignoreErrors: boolean) => string | null | void;
//# sourceMappingURL=get-format.d.ts.map