/**
 * SVG viewBox: x, y, width, height
 */
type SVGViewBox = [x: number, y: number, width: number, height: number];
/**
 * Get viewBox from string
 */
declare function getSVGViewBox(value: string): SVGViewBox | undefined;
export { SVGViewBox, getSVGViewBox };